###############################################################################
# (c) 2005-2020 Copyright, Real-Time Innovations.  All rights reserved.       #
# No duplications, whole or partial, manual or electronic, may be made        #
# without express written permission.  Any such copies, or revisions thereof, #
# must display this notice unaltered.                                         #
# This code contains trade secrets of Real-Time Innovations, Inc.             #
###############################################################################

from __future__ import print_function

# Updating the system path is not required if you have pip-installed
# rticonnextdds-connector
from sys import path as sys_path
from os import path as os_path
file_path = os_path.dirname(os_path.realpath(__file__))
sys_path.append(file_path + "/../../../")

import rticonnextdds_connector as rti
import ctypes
import numpy as np

ARRAY_LEN = 1024
# Expose the DynamicData API which we are going to use
# It has the following prototype:
# DDS_ReturnCode_t DDS_DynamicData_get_octet_array(
#       DDS_DynamicData * self,
#       DDS_Octet *array,
#       DDS_UnsignedLong *length,
#       const char *name,
#       DDS_DynamicDataMemberId member_id)
# On Windows we would need to explicitly load the nddsc.dll
# nddsc_library = cdll.LoadLibrary("C:\\Path\\To\\nddsc.dll")
# And then use nddsc_library variable above instead of
# rti.connector_binding.library
DDS_DynamicData_get_octet_array = rti.connector_binding.library.DDS_DynamicData_get_octet_array
DDS_DynamicData_get_octet_array.restype = ctypes.c_int
DDS_DynamicData_get_octet_array.argtypes = [ctypes.c_void_p, ctypes.c_void_p, ctypes.c_void_p, ctypes.c_char_p, ctypes.c_ulong]

def getSequenceNatively(native_dynamic_data, field_name):
    # Now obtain the sequence requested (described by field_name)
    # Unsigned long for the expected length of the array
    array_length = ctypes.c_ulong(ARRAY_LEN)
    # Create a new "type" compatible with the sequence we are obtaining
    in_array_type = ctypes.c_char * ARRAY_LEN
    # Create an instance of that type which will we pass to the native API
    in_array = in_array_type()
    DDS_DynamicData_get_octet_array(
            ctypes.cast(native_dynamic_data, ctypes.c_void_p),
            ctypes.byref(in_array),
            ctypes.byref(array_length),
            field_name.encode("utf8"),
            0)
    # This uses numpy. If numpy isn't available, you could do
    # return [in_array[i] for i in xrange(ARRAY_LEN)]
    return np.ndarray((ARRAY_LEN, ), np.ubyte, in_array, order='C')

with rti.open_connector(
        config_name="MyParticipantLibrary::MySubParticipant",
        url=file_path + "/../Example.xml") as connector:

    input = connector.get_input("MySubscriber::MyImageReader")
    input.wait_for_publications()

    for i in range(1, 500):
        input.wait()
        input.take()
        for sample in input.samples.valid_data_iter:
            r_data = getSequenceNatively(sample.native, "r_data")
            g_data = getSequenceNatively(sample.native, "g_data")
            b_data = getSequenceNatively(sample.native, "b_data")
